@@ -2,7 +2,7 @@
  * @file
  *
  * @author G. Kaplan and A. Kovacs
- * @version 1.3.0
+ * @version 1.4.2
  *
  *  SuperNOVAS astrometry software based on the Naval Observatory Vector Astrometry Software (NOVAS).
  *  It has been modified to fix outstanding issues and to make it easier to use.
@@ -65,7 +65,7 @@
 #define SUPERNOVAS_MINOR_VERSION  4
 
 /// Integer sub version of the release
-#define SUPERNOVAS_PATCHLEVEL     1
+#define SUPERNOVAS_PATCHLEVEL     2
 
 /// Additional release information in version, e.g. "-1", or "-rc1", or empty string "" for releases.
 #define SUPERNOVAS_RELEASE_STRING ""
@@ -678,15 +678,15 @@ enum novas_cio_location_type {
 
 #ifndef DEFAULT_CIO_LOCATOR_FILE
 #  if COMPAT
-     /// Path / name of file to use for interpolating the CIO location relative to GCRS
-     /// This file can be generated with the <code>cio_file.c</code> tool using the
-     /// <code>CIO_RA.TXT</code> data (both are included in the distribution)
+/// Path / name of file to use for interpolating the CIO location relative to GCRS
+/// This file can be generated with the <code>cio_file.c</code> tool using the
+/// <code>CIO_RA.TXT</code> data (both are included in the distribution)
 #    define DEFAULT_CIO_LOCATOR_FILE      "cio_ra.bin"
 #  else
 /// Path / name of file to use for interpolating the CIO location relative to GCRS
 /// This file can be generated with the <code>cio_file.c</code> tool using the
 /// <code>CIO_RA.TXT</code> data (both are included in the distribution)
-#    define DEFAULT_CIO_LOCATOR_FILE      "/usr/share/novas/cio_ra.bin"
+#    define DEFAULT_CIO_LOCATOR_FILE      "/usr/share/supernovas/cio_ra.bin"
 #  endif
 #endif
 
