@@ -233,13 +233,16 @@ typedef int (*novas_ephem_provider)(const char *name, long id, double jd_tdb_hig
  *                      is responsible for calling free() on the returned value when it is no
  *                      longer needed.
  *
+ * @deprecated (<i>legacy function</i>) Use `set_ephem_provider()` instead to specify what
+ *             function should be used to calculate ephemeris positions for Solar-system
+ *             objects. This prototype is provided to extend support for legacy NOVAS C
+ *             applications only. In NOVAS, the function had to be user defined, as a custom
+ *             implementation.
+ *
  * @sa set_ephem_provider()
  * @sa novas_ephem_provider
  * @sa ephemeris()
  * @sa NOVAS_EPHEM_OBJECT
- *
- * @since 1.0
- * @author Attila Kovacs
  */
 double *readeph(int mp, const char *restrict name, double jd_tdb, int *restrict error);
 
@@ -282,6 +285,12 @@ novas_ephem_provider get_ephem_provider();
  *                      range, 2 if 'body' is invalid, or 3 if the ephemeris data cannot be
  *                      produced for other reasons.
  *
+ * @deprecated (<i>legacy function</i>) Use `set_planet_provider()` instead to specify what
+ *             function should be used to calculate ephemeris positions for major planets. This
+ *             function is provided to extend support for legacy NOVAS C applications only. In
+ *             NOVAS, the function had to be user defined, either by linking against a
+ *             `solsys[1-3].c` module, or by providing a custom user implementation.
+ *
  * @sa novas_planet
  * @sa solarsystem_hp()
  * @sa set_planet_provider()
@@ -322,6 +331,12 @@ short solarsystem(double jd_tdb, short body, short origin, double *restrict posi
  *                      (errno set to ENOSYS), or some other error code (NOVAS C was not very
  *                      consistent here...)
  *
+ * @deprecated (<i>legacy function</i>) Use `set_planet_provider_hp()` instead to specify what
+ *             function should be used to calculate high-precision ephemeris positions for major
+ *             planets. This function is provided to extend support for legacy NOVAS C
+ *             applications only. In NOVAS, the function had to be user defined, either by linking
+ *             against a `solsys[1-3].c` module, or by providing a custom user implementation.
+ *
  * @sa solarsystem()
  * @sa set_planet_provider_hp()
  * @sa place()
@@ -370,15 +385,48 @@ long novas_to_naif_planet(enum novas_planet id);
 
 long novas_to_dexxx_planet(enum novas_planet id);
 
+enum novas_planet novas_planet_for_name(const char *restrict name);
+
 
 // Added in v1.3 --------------------------------->
 
+// in orbital.c
+int novas_set_orbsys_pole(enum novas_reference_system type, double ra, double dec, novas_orbital_system *restrict sys);
+
+int make_orbital_object(const char *name, long num, const novas_orbital *orbit, object *body);
+
+int novas_orbit_posvel(double jd_tdb, const novas_orbital *restrict orbit, enum novas_accuracy accuracy,
+        double *restrict pos, double *restrict vel);
+
 double novas_helio_dist(double jd_tdb, const object *restrict source, double *restrict rate);
 
 double novas_solar_power(double jd_tdb, const object *restrict source);
 
 double novas_solar_illum(const object *restrict source, const novas_frame *restrict frame);
 
+double novas_sun_angle(const object *restrict source, const novas_frame *restrict frame);
+
+double novas_moon_angle(const object *restrict source, const novas_frame *restrict frame);
+
+
+// Added in v1.4 --------------------------------->
+
+// in orbit.c
+int novas_orbit_native_posvel(double jd_tdb, const novas_orbital *restrict orbit, double *restrict pos, double *restrict vel);
+
+// in plorbit.c
+int novas_make_planet_orbit(enum novas_planet id, double jd_tdb, novas_orbital *restrict orbit);
+
+int novas_make_moon_orbit(double jd_tdb, novas_orbital *restrict orbit);
+
+int novas_approx_heliocentric(enum novas_planet id, double jd_tdb, double *restrict pos, double *restrict vel);
+
+int novas_approx_sky_pos(enum novas_planet id, const novas_frame *restrict frame, enum novas_reference_system sys, sky_pos *restrict out);
+
+double novas_moon_phase(double jd_tdb);
+
+double novas_next_moon_phase(double phase, double jd_tdb);
+
 
 /// \cond PRIVATE
 
