@@ -15,88 +15,6 @@
 #include "novas.h"
 /// \endcond
 
-/**
- * Celestial pole offset &psi; for high-precision applications. It was visible to users in NOVAS C 3.1,
- * hence we continue to expose it also for back compatibility.
- *
- * @sa EPS_COR
- * @sa cel_pole()
- */
-double PSI_COR = 0.0;
-
-/**
- * Celestial pole offset &epsilon; for high-precision applications. It was visible to users in NOVAS C 3.1,
- * hence we continue to expose it also for back compatibility.
- *
- * @sa PSI_COR
- * @sa cel_pole()
- */
-double EPS_COR = 0.0;
-
-/// \cond PRIVATE
-
-/**
- * Converts <i>dx,dy</i> pole offsets to d&psi; d&epsilon;. The former is in GCRS, vs the latter in
- * True of Date (TOD) -- and note the different units!
- *
- * NOTES:
- * <ol>
- * <li>The current UT1 - UTC time difference, and polar offsets, historical data and near-term
- * projections are published in the
- * <a href="https://www.iers.org/IERS/EN/Publications/Bulletins/bulletins.html>IERS Bulletins</a>
- * </li>
- * </ol>
- *
- * REFERENCES:
- * <ol>
- *  <li>Kaplan, G. (2005), US Naval Observatory Circular 179.</li>
- *  <li>Kaplan, G. (2003), USNO/AA Technical Note 2003-03.</li>
- * </ol>
- *
- * @param jd_tt       [day] Terrestrial Time (TT) based Julian Date.
- * @param dx          [mas] Earth orientation: GCRS pole offset dx, e.g. as published by IERS Bulletin A.
- * @param dy          [mas] Earth orientation: GCRS pole offset dy, e.g. as published by IERS Bulletin A.
- * @param[out] dpsi   [arcsec] Calculated TOD orientation d&psi;.
- * @param[out] deps   [arcsec] Calculated TOD orientation d&epsilon;.
- * @return            0
- *
- * @sa cel_pole()
- *
- * @since 1.1
- * @author Attila Kovacs
- */
-int polar_dxdy_to_dpsideps(double jd_tt, double dx, double dy, double *restrict dpsi, double *restrict deps) {
-  // Components of modeled pole unit vector referred to GCRS axes, that is, dx and dy.
-  const double t = (jd_tt - JD_J2000) / JULIAN_CENTURY_DAYS;
-
-  // The following algorithm, to transform dx and dy to
-  // delta-delta-psi and delta-delta-epsilon, is from eqs. (7)-(9) of the
-  // second reference.
-  //
-  // Trivial model of pole trajectory in GCRS allows computation of dz.
-  const double x = (2004.190 * t) * ARCSEC;
-  const double dz = -(x + 0.5 * x * x * x) * dx;
-
-  // Form pole offset vector (observed - modeled) in GCRS.
-  double dp[3] = { dx * MAS, dy * MAS, dz * MAS };
-
-  // Precess pole offset vector to mean equator and equinox of date.
-  gcrs_to_mod(jd_tt, dp, dp);
-
-  // Compute delta-delta-psi and delta-delta-epsilon in arcseconds.
-  if(dpsi) {
-    // Compute sin_e of mean obliquity of date.
-    const double sin_e = sin(mean_obliq(jd_tt) * ARCSEC);
-    *dpsi = (dp[0] / sin_e) / ARCSEC;
-  }
-  if(deps)
-    *deps = dp[1] / ARCSEC;
-
-  return 0;
-}
-/// \endcond
-
-/// ==========================================================================
 
 /**
  * Computes the position and velocity vectors of a terrestrial observer with respect to the
@@ -189,79 +107,7 @@ int terra(const on_surface *restrict location, double lst, double *restrict pos,
   return 0;
 }
 
-/**
- * Computes quantities related to the orientation of the Earth's rotation axis at Julian date 'jd_tdb'.
- *
- *  Values of the celestial pole offsets 'PSI_COR' and 'EPS_COR' are set using function 'cel_pole', if
- *  desired.  See the prolog of cel_pole() for details.
- *
- * NOTES:
- * <ol>
- * <li>This function caches the results of the last calculation in case it may be re-used at
- * no extra computational cost for the next call.</li>
- * </ol>
- *
- * @param jd_tdb    [day] Barycentric Dynamical Time (TDB) based Julian date.
- * @param accuracy  NOVAS_FULL_ACCURACY (0) or NOVAS_REDUCED_ACCURACY (1)
- * @param[out] mobl      [deg] Mean obliquity of the ecliptic in degrees. It may be NULL if not required.
- * @param[out] tobl      [deg] True obliquity of the ecliptic in degrees. It may be NULL if not required.
- * @param[out] ee        [deg] Equation of the equinoxes in seconds of time. It may be NULL if not required.
- * @param[out] dpsi      [arcsec] Nutation in longitude in arcseconds. It may be NULL if not required.
- * @param[out] deps      [arcsec] Nutation in obliquity in arcseconds. It may be NULL if not required.
- *
- * @return          0 if successful, or -1 if the accuracy argument is invalid
- *
- * @sa cel_pole()
- * @sa place()
- * @sa equ2ecl()
- * @sa ecl2equ()
- */
-int e_tilt(double jd_tdb, enum novas_accuracy accuracy, double *restrict mobl, double *restrict tobl,
-        double *restrict ee, double *restrict dpsi, double *restrict deps) {
-  static THREAD_LOCAL enum novas_accuracy acc_last = -1;
-  static THREAD_LOCAL double jd_last = NAN;
-  static THREAD_LOCAL double d_psi, d_eps, mean_ob, true_ob, eqeq;
 
-  if(accuracy != NOVAS_FULL_ACCURACY && accuracy != NOVAS_REDUCED_ACCURACY)
-    return novas_error(-1, EINVAL, "e_tilt", "invalid accuracy: %d", accuracy);
-
-  // Compute the nutation angles (arcseconds) if the input Julian date
-  // is significantly different from the last Julian date, or the
-  // accuracy mode has changed from the last call.
-  if(!novas_time_equals(jd_tdb, jd_last) || (accuracy != acc_last)) {
-    // Compute time in Julian centuries from epoch J2000.0.
-    const double t = (jd_tdb - JD_J2000) / JULIAN_CENTURY_DAYS;
-
-    nutation_angles(t, accuracy, &d_psi, &d_eps);
-
-    // Compute mean obliquity of the ecliptic in degrees.
-    mean_ob = mean_obliq(jd_tdb) / 3600.0;
-
-    // Obtain complementary terms for equation of the equinoxes in arcseconds.
-    eqeq = (d_psi * cos(mean_ob * DEGREE) + ee_ct(jd_tdb, 0.0, accuracy) / ARCSEC) / 15.0;
-
-    // Compute true obliquity of the ecliptic in degrees.
-    true_ob = mean_ob + d_eps / 3600.0;
-
-    // Reset the values of the last Julian date and last mode.
-    jd_last = jd_tdb;
-    acc_last = accuracy;
-  }
-
-  // Set output values.
-  if(dpsi)
-    *dpsi = d_psi + PSI_COR;
-  if(deps)
-    *deps = d_eps + EPS_COR;
-  if(ee)
-    *ee = eqeq;
-  if(mobl)
-    *mobl = mean_ob;
-  if(tobl)
-    *tobl = true_ob;
-
-  return 0;
-}
 
 /**
  * Computes the Greenwich sidereal time, either mean or apparent, at the specified Julian date.
@@ -296,6 +142,7 @@ int e_tilt(double jd_tdb, enum novas_accuracy accuracy, double *restrict mobl, d
  *                    invalid 2 if 'method' is invalid, or else 10--30 with 10 + the error from
  *                    cio_location().
  *
+ * @sa novas_time_gst()
  * @sa era()
  * @sa tod_to_itrs()
  * @sa itrs_to_tod()
@@ -408,11 +255,11 @@ short sidereal_time(double jd_ut1_high, double jd_ut1_low, double ut1_to_tt, enu
  * JD_J2000), but it avoids many two-PI 'wraps' that decrease precision (adopted from SOFA Fortran
  * routine iau_era00; see also expression at top of page 35 of IERS Conventions (1996)).
  *
- *  REFERENCES:
- *  <ol>
- *   <li>IAU Resolution B1.8, adopted at the 2000 IAU General Assembly, Manchester, UK.</li>
- *   <li>Kaplan, G. (2005), US Naval Observatory Circular 179.</li>
- *  </ol>
+ * REFERENCES:
+ * <ol>
+ *  <li>IAU Resolution B1.8, adopted at the 2000 IAU General Assembly, Manchester, UK.</li>
+ *  <li>Kaplan, G. (2005), US Naval Observatory Circular 179.</li>
+ * </ol>
  *
  * @param jd_ut1_high   [day] High-order part of UT1 Julian date.
  * @param jd_ut1_low    [day] Low-order part of UT1 Julian date.
@@ -436,93 +283,38 @@ double era(double jd_ut1_high, double jd_ut1_low) {
   return theta;
 }
 
-/**
- * specifies the celestial pole offsets for high-precision applications.  Each set of offsets is
- * a correction to the modeled position of the pole for a specific date, derived from observations
- * and published by the IERS.
- *
- * The variables 'PSI_COR' and 'EPS_COR' are used only in NOVAS function e_tilt().
- *
- * This function, if used, should be called before any other NOVAS functions for a given date.
- * Values of the pole offsets specified via a call to this function will be used until explicitly
- * changed.
- *
- * 'tjd' is used only if 'type' is POLE_OFFSETS_X_Y (2), to transform dx and dy to the equivalent
- * &Delta;&delta;&psi; and &Delta;&delta;&epsilon; values.
- *
- * If 'type' is POLE_OFFSETS_X_Y (2), dx and dy are unit vector component corrections, but are
- * expressed in milliarcseconds simply by multiplying by 206264806, the number of milliarcseconds
- * in one radian.
- *
- * NOTES:
- * <ol>
- * <li>The current UT1 - UTC time difference, and polar offsets, historical data and near-term
- * projections are published in the
- * <a href="https://www.iers.org/IERS/EN/Publications/Bulletins/bulletins.html>IERS Bulletins</a>
- * </li>
- * </ol>
- *
- * REFERENCES:
- * <ol>
- *  <li>Kaplan, G. (2005), US Naval Observatory Circular 179.</li>
- *  <li>Kaplan, G. (2003), USNO/AA Technical Note 2003-03.</li>
- * </ol>
- *
- * @param jd_tt     [day] Terrestrial Time (TT) based Julian date.
- * @param type      POLE_OFFSETS_DPSI_DEPS (1) or POLE_OFFSETS_X_Y (2)
- * @param dpole1    [mas] Value of celestial pole offset in first coordinate, (&Delta;&delta;&psi;
- *                  or dx) in milliarcseconds.
- * @param dpole2    [mas] Value of celestial pole offset in second coordinate, (&Delta;&delta;&epsilon;
- *                  or dy) in milliarcseconds.
- * @return          0 if successful, or else 1 if 'type' is invalid.
- *
- * @sa cirs_to_itrs()
- * @sa tod_to_itrs()
- * @sa e_tilt()
- * @sa place()
- * @sa get_ut1_to_tt()
- * @sa sidereal_time()
- * @sa NOVAS_FULL_ACCURACY
- */
-short cel_pole(double jd_tt, enum novas_pole_offset_type type, double dpole1, double dpole2) {
-  switch(type) {
-    case POLE_OFFSETS_DPSI_DEPS:
-
-      // Angular coordinates of modeled pole referred to mean ecliptic of
-      // date, that is,delta-delta-psi and delta-delta-epsilon.
-      PSI_COR = dpole1 * 1.0e-3;
-      EPS_COR = dpole2 * 1.0e-3;
-      break;
-
-    case POLE_OFFSETS_X_Y: {
-      polar_dxdy_to_dpsideps(jd_tt, dpole1, dpole2, &PSI_COR, &EPS_COR);
-      break;
-    }
-
-    default:
-      return novas_error(1, EINVAL, "cel_pole", "invalid polar offset type: %d", type);
-  }
-
-  return 0;
-}
 
 /**
  * Corrects a vector in the ITRS (rotating Earth-fixed system) for polar motion, and also
  * corrects the longitude origin (by a tiny amount) to the Terrestrial Intermediate Origin
- * (TIO).  The ITRS vector is thereby transformed to the terrestrial intermediate reference
- * system (TIRS) or Pseudo Earth Fixed (PEF), based on the true (rotational) equator and TIO;
+ * (TIO).  The ITRS vector is thereby transformed to the Terrestrial Intermediate Reference
+ * System (TIRS) or Pseudo Earth Fixed (PEF), based on the true (rotational) equator and TIO;
  * or vice versa.  Because the true equator is the plane orthogonal to the direction of the
  * Celestial Intermediate Pole (CIP), the components of the output vector are referred to z
  * and x axes toward the CIP and TIO, respectively.
  *
+ * NOTES:
+ * <ol>
+ * <li>Generally, this function should not be called if global pole offsets were set via
+ * cel_pole() and then used via place() or one of its variants to calculate Earth orientation
+ * corrected (TOD or CIRS) apparent coordinates. In such cases, calling wobble() would apply
+ * duplicate corrections. It is generally best to forgo using cel_pole() going forward, and
+ * instead apply Earth orinetation corrections with wobble() only when converting vectors to the
+ * Earth-fixed ITRS frame.</li>
+ * </ol>
+ *
  * REFERENCES:
  * <ol>
  * <li>Kaplan, G. H. et. al. (1989). Astron. Journ. 97, 1197-1210.</li>
- * <li>Lambert & Bizouard (2002), Astronomy and Astrophysics 394, 317-321.</li>
+ * <li>Lambert &amp; Bizouard (2002), Astronomy and Astrophysics 394, 317-321.</li>
  * </ol>
  *
  * @param jd_tt         [day] Terrestrial Time (TT) based Julian date.
- * @param direction     WOBBLE_ITRS_TO_PEF (0) or WOBBLE_PEF_TO_ITRS (nonzero)
+ * @param direction     WOBBLE_ITRS_TO_TIRS (0) or WOBBLE_TIRS_TO_ITRS (1) to include corrections
+ *                      for the TIO's longitude (IAU 2006 method); or else
+ *                      WOBBLE_ITRS_TO_PEF (2) or WOBBLE_PEF_TO_ITRS (3) to correct for dx, dy but
+ *                      not for the TIO's longitude (old, pre IAU 2006 method). Negative values
+ *                      default to WOBBLE_TIRS_TO_ITRS.
  * @param xp            [arcsec] Conventionally-defined X coordinate of Celestial Intermediate
  *                      Pole with respect to ITRS pole, in arcseconds.
  * @param yp            [arcsec] Conventionally-defined Y coordinate of Celestial Intermediate
@@ -533,7 +325,7 @@ short cel_pole(double jd_tt, enum novas_pole_offset_type type, double dpole1, do
  *                      in the final system defined by 'direction'. It can be the same vector
  *                      as the input.
  *
- * @return              0 if successful, or -1 if the output vector argument is NULL.
+ * @return              0 if successful, or -1 if the direction is invalid output vector argument is NULL.
  *
  * @sa cel_pole()
  * @sa cirs_to_itrs()
@@ -544,35 +336,46 @@ short cel_pole(double jd_tt, enum novas_pole_offset_type type, double dpole1, do
  * @sa NOVAS_FULL_ACCURACY
  */
 int wobble(double jd_tt, enum novas_wobble_direction direction, double xp, double yp, const double *in, double *out) {
-  double xpole, ypole, t, s1, y1;
+  static const char *fn = "wobble";
+
+  double s1 = 0.0;
+
+  if((short) direction < 0)
+    direction = 1;
+  else if(direction >= NOVAS_WOBBLE_DIRECTIONS)
+    return novas_error(-1, EINVAL, fn, "invalid direction: %d", direction);
 
   if(!in || !out)
-    return novas_error(-1, EINVAL, "wobble", "NULL input or output 3-vector: in=%p, out=%p", in, out);
+    return novas_error(-1, EINVAL, fn, "NULL input or output 3-vector: in=%p, out=%p", in, out);
 
-  xpole = xp * ARCSEC;
-  ypole = yp * ARCSEC;
+  xp *= ARCSEC;
+  yp *= ARCSEC;
 
   // Compute approximate longitude of TIO (s'), using eq. (10) of the second reference
-  t = (jd_tt - JD_J2000) / JULIAN_CENTURY_DAYS;
-  s1 = -47.0e-6 * ARCSEC * t;
-
-  y1 = in[1];
+  if(direction == WOBBLE_ITRS_TO_TIRS || direction == WOBBLE_TIRS_TO_ITRS) {
+    double t = (jd_tt - JD_J2000) / JULIAN_CENTURY_DAYS;
+    s1 = -47.0e-6 * ARCSEC * t;
+  }
 
   // Compute elements of rotation matrix.
   // Equivalent to R3(-s')R2(x)R1(y) as per IERS Conventions (2003).
-  if(direction == WOBBLE_ITRS_TO_PEF)
-    novas_tiny_rotate(in, -ypole, -xpole, s1, out);
-  else
-    novas_tiny_rotate(in, ypole, xpole, -s1, out);
-
-  // Second-order correction for the non-negligible xp, yp product...
-  out[0] += xpole * ypole * y1;
+  if(direction == WOBBLE_ITRS_TO_TIRS || direction == WOBBLE_ITRS_TO_PEF) {
+    double y = in[1];
+    novas_tiny_rotate(in, -yp, -xp, s1, out);
+    // Second-order correction for the non-negligible xp, yp product...
+    out[0] += xp * yp * y;
+  }
+  else {
+    double x = in[0];
+    novas_tiny_rotate(in, yp, xp, -s1, out);
+    out[1] += xp * yp * x;
+  }
 
   return 0;
 }
 
 /**
- * Computes the geocentric position and velocity of an observer. The final vectors are expressed in the GCRS.
+ * Computes the geocentric GCRS position and velocity of an observer.
  *
  * @param jd_tt       [day] Terrestrial Time (TT) based Julian date.
  * @param ut1_to_tt   [s] TT - UT1 time difference in seconds
@@ -772,3 +575,4 @@ int limb_angle(const double *pos_src, const double *pos_obs, double *restrict li
 
   return 0;
 }
+
