@@ -41,9 +41,9 @@
 /// \endcond
 
 static int cmp_sys(enum novas_reference_system a, enum novas_reference_system b) {
-  // GCRS=0, TOD=1, CIRS=2, ICRS=3, J2000=4, MOD=5
-  // TOD->-3, MOD->-2, J2000->-1, GCRS/ICRS->0, CIRS->1
-  static const int index[] = { 0, -3, 1, 0, -1, -2 };
+  // GCRS=0, TOD=1, CIRS=2, ICRS=3, J2000=4, MOD=5, TIRS=6, ITRS=7
+  // TOD->-3, MOD->-2, J2000->-1, GCRS/ICRS->0, CIRS->1, TIRS->2, ITRS->3
+  static const int index[] = { 0, -3, 1, 0, -1, -2, 2, 3 };
 
   if(a < 0 || a >= NOVAS_REFERENCE_SYSTEMS)
     return novas_error(-2, EINVAL, "cmp_sys", "Invalid reference system (#1): %d", a);
@@ -69,7 +69,6 @@ static int matrix_transform(const double *in, const novas_matrix *matrix, double
   return 0;
 }
 
-
 static int matrix_inv_rotate(const double *in, const novas_matrix *matrix, double *out) {
   // IMPORTANT! use only with unitary matrices.
   double v[3];
@@ -161,6 +160,15 @@ static int set_precession(novas_frame *frame) {
   const double omegaa = ((((+0.0000003337 * t - 0.000000467) * t - 0.00772503) * t + 0.0512623) * t - 0.025754) * t * ARCSEC + eps0;
   const double chia = ((((-0.0000000560 * t + 0.000170663) * t - 0.00121197) * t - 2.3814292) * t + 10.556403) * ARCSEC * t;
 
+  // P03rev2 / Capitaine at al. (2005) eqs. (11)
+  //const double psia = t * (5038.482090 + t * (-1.0789921 + t * (-0.00114040 + t * (0.000132851 - t * 0.0000000951)))) * ARCSEC;
+  //const double omegaa = eps0 + t * (-0.025675 + t * (0.0512622 + t * (-0.00772501 + t * (-0.000000467 + t * 0.0000003337)))) * ARCSEC;
+
+  // Liu & Capitaine (2017)
+  //const double chia = t * (10.556240 + t * (-2.3813876 + t * (-0.00121311 + t * (0.000160286 + t * 0.000000086)))) * ARCSEC;
+  //const double psia = t * (5038.481270 + t * (-1.0732468 + t * (0.01573403 + t * (0.000127135 - t * 0.0000001020)))) * ARCSEC;
+  //const double omegaa = eps0 + t * (-0.024725 + t * (0.0512626 + t * (-0.0077249 + t * (-0.000000267 + t * 0.000000267)))) * ARCSEC;
+
   const double sa = sin(eps0);
   const double ca = cos(eps0);
   const double sb = sin(-psia);
@@ -221,6 +229,54 @@ static int set_nutation(novas_frame *frame) {
   return 0;
 }
 
+static int set_spin_matrix(double angle, novas_matrix *T) {
+  double c, s;
+
+  memset(T, 0, sizeof(novas_matrix));
+
+  angle *= DEGREE;
+  c = cos(angle);
+  s = sin(angle);
+
+  // Rotation matrix (non-zero elements only).
+  T->M[0][0] = c;
+  T->M[0][1] = s;
+  T->M[1][0] = -s;
+  T->M[1][1] = c;
+  T->M[2][2] = 1.0;
+
+  return 0;
+}
+
+static int set_wobble_matrix(const novas_frame *frame, novas_matrix *T) {
+  // TIRS / PEF -> ITRS
+
+  // Compute approximate longitude of TIO (s'), using eq. (10) of the second reference
+  const double t = (frame->time.ijd_tt + frame->time.fjd_tt - JD_J2000) / JULIAN_CENTURY_DAYS;
+
+  const double ax = frame->dy * MAS;
+  const double ay = frame->dx * MAS;
+  const double az = 47.0e-6 * ARCSEC * t;   // -s1
+
+  const double A[3] = { ax * ax, ay * ay, az * az };
+
+  //memset(T, 0, sizeof(novas_matrix));
+
+  T->M[0][0] = 1.0 - 0.5 * (A[1] + A[2]);
+  T->M[0][1] = -az;
+  T->M[0][2] = ay;
+
+  T->M[1][0] = az + (ax * ay);
+  T->M[1][1] = 1.0 - 0.5 * (A[0] + A[2]);
+  T->M[1][2] = -ax;
+
+  T->M[2][0] = -ay;
+  T->M[2][1] = ax;
+  T->M[2][2] = 1.0 - 0.5 * (A[0] + A[1]);
+
+  return 0;
+}
+
 static int set_obs_posvel(novas_frame *frame) {
   int res = obs_posvel(novas_get_time(&frame->time, NOVAS_TDB), frame->time.ut1_to_tt, frame->accuracy, &frame->observer,
           frame->earth_pos, frame->earth_vel, frame->obs_pos, frame->obs_vel);
@@ -274,11 +330,21 @@ static int frame_aberration(const novas_frame *frame, int dir, double *pos) {
   return novas_error(-1, ECANCELED, "frame_aberration", "failed to converge");
 }
 
-
-static int is_frame_initialized(const novas_frame *frame) {
-  if(!frame) return 0;
+/// \cond PRIVATE
+/**
+ * Checks if a frame has been initialized either via a call to `make_frame()`.
+ *
+ * @param frame   The observing frame, defining the time of observation and the observer location.
+ *                !!! It cannot be NULL !!!
+ * @return        boolean TRUE (1) if the frame has been initialized, or else FALSE (0).
+ *
+ * @sa make_frame()
+ */
+int novas_frame_is_initialized(const novas_frame *frame) {
+  //if(!frame) return 0;
   return frame->state == FRAME_INITIALIZED;
 }
+/// \endcond
 
 /**
  * Sets up a observing frame for a specific observer location, time of observation, and accuracy
@@ -294,13 +360,22 @@ static int is_frame_initialized(const novas_frame *frame) {
  * provider for the major planets will result in an error in the 10--40 range from the required
  * ephemeris() call.
  *
+ * NOTES:
+ * <ol>
+ * <li>This function expects the Earth polar wobble parameters to be defined on a per-frame basis
+ * and will not use the legacy global (undated) orientation parameters set via cel_pole().</li>
+ * </ol>
  *
  * @param accuracy    Accuracy requirement, NOVAS_FULL_ACCURACY (0) for the utmost precision or
  *                    NOVAS_REDUCED_ACCURACY (1) if ~1 mas accuracy is sufficient.
  * @param obs         Observer location
  * @param time        Time of observation
- * @param dx          [mas] Earth orientation parameter, polar offset in x.
- * @param dy          [mas] Earth orientation parameter, polar offset in y.
+ * @param dx          [mas] Earth orientation parameter, polar offset in x, e.g. from the IERS
+ *                    Bulletins. (The global, undated value set by cel_pole() is not not used
+ *                    here.) You can use 0.0 if sub-arcsecond accuracy is not required.
+ * @param dy          [mas] Earth orientation parameter, polar offset in y, e.g. from the IERS
+ *                    Bulletins. (The global, undated value set by cel_pole() is not not used
+ *                    here.) You can use 0.0 if sub-arcsecond accuracy is not required.
  * @param[out] frame  Pointer to the observing frame to configure.
  * @return            0 if successful,
  *                    10--40: error is 10 + the error from ephemeris(),
@@ -329,7 +404,7 @@ int novas_make_frame(enum novas_accuracy accuracy, const observer *obs, const no
   static const object sun = NOVAS_SUN_INIT;
 
   double tdb2[2];
-  double mobl, tobl, ee, dpsi, deps;
+  double dpsi, deps;
   long ijd_ut1;
   double fjd_ut1;
 
@@ -351,20 +426,23 @@ int novas_make_frame(enum novas_accuracy accuracy, const observer *obs, const no
   tdb2[0] = time->ijd_tt;
   tdb2[1] = time->fjd_tt + tt2tdb(time->ijd_tt + time->fjd_tt) / DAY;
 
-  // Various calculated quantities for frame transformations
-  e_tilt(tdb2[0] + tdb2[1], frame->accuracy, &mobl, &tobl, &ee, NULL, NULL);
-
-  frame->mobl = mobl * DEGREE;
-  frame->tobl = tobl * DEGREE;
-  frame->ee = ee * DEGREE;
-
   nutation_angles((tdb2[0] + tdb2[1] - NOVAS_JD_J2000) / JULIAN_CENTURY_DAYS, accuracy, &dpsi, &deps);
 
+  // dpsi0 / dpes0 w/o the global pole offsets set via cel_pole()
   frame->dpsi0 = dpsi * ARCSEC;
   frame->deps0 = deps * ARCSEC;
   frame->dx = dx;
   frame->dy = dy;
 
+  // Compute mean obliquity of the ecliptic in degrees.
+  frame->mobl = mean_obliq(tdb2[0] + tdb2[1]) * ARCSEC;
+
+  // Obtain complementary terms for equation of the equinoxes in seconds of time.
+  frame->ee = dpsi * ARCSEC * cos(frame->mobl) + ee_ct(time->ijd_tt, time->fjd_tt, accuracy);
+
+  // Compute true obliquity of the ecliptic in degrees.
+  frame->tobl = frame->mobl + deps * ARCSEC;
+
   fjd_ut1 = novas_get_split_time(time, NOVAS_UT1, &ijd_ut1);
   frame->era = era(ijd_ut1, fjd_ut1);
 
@@ -410,7 +488,7 @@ int novas_change_observer(const novas_frame *orig, const observer *obs, novas_fr
   if(!orig || !obs || !out)
     return novas_error(-1, EINVAL, fn, "NULL parameter: orig=%p, obs=%p, out=%p", orig, obs, out);
 
-  if(!is_frame_initialized(orig))
+  if(!novas_frame_is_initialized(orig))
     return novas_error(-1, EINVAL, fn, "frame at %p not initialized", out);
 
   if(out != orig)
@@ -430,14 +508,22 @@ int novas_change_observer(const novas_frame *orig, const observer *obs, novas_fr
   return 0;
 }
 
-static int icrs_to_sys(const novas_frame *frame, double *pos, enum novas_reference_system sys) {
+static int icrs_to_sys(const novas_frame *restrict frame, double *restrict pos, enum novas_reference_system sys) {
   switch(sys) {
     case NOVAS_ICRS:
     case NOVAS_GCRS:
       return 0;
 
     case NOVAS_CIRS:
+    case NOVAS_TIRS:
+    case NOVAS_ITRS:
       matrix_transform(pos, &frame->gcrs_to_cirs, pos);
+      if(sys == NOVAS_CIRS) return 0;
+
+      spin(frame->era, pos, pos);
+      if(sys == NOVAS_TIRS) return 0;
+
+      wobble(frame->time.ijd_tt + frame->time.fjd_tt, WOBBLE_TIRS_TO_ITRS, frame->dx, frame->dy, pos, pos);
       return 0;
 
     case NOVAS_J2000:
@@ -521,7 +607,7 @@ int novas_geom_posvel(const object *restrict source, const novas_frame *restrict
   if(pos == vel)
     return novas_error(-1, EINVAL, fn, "identical output pos and vel 3-vectors @ %p.", pos);
 
-  if(!is_frame_initialized(frame))
+  if(!novas_frame_is_initialized(frame))
     return novas_error(-1, EINVAL, fn, "frame at %p not initialized", frame);
 
   if(frame->accuracy != NOVAS_FULL_ACCURACY && frame->accuracy != NOVAS_REDUCED_ACCURACY)
@@ -575,10 +661,9 @@ int novas_geom_posvel(const object *restrict source, const novas_frame *restrict
   return 0;
 }
 
-
 /**
  * Calculates an apparent location on sky for the source. The position takes into account the
- * proper motion (for sidereal soure), or is antedated for light-travel time (for Solar-System
+ * proper motion (for sidereal source), or is antedated for light-travel time (for Solar-System
  * bodies). It also applies an appropriate aberration correction and gravitational deflection of
  * the light.
  *
@@ -615,8 +700,8 @@ int novas_geom_posvel(const object *restrict source, const novas_frame *restrict
  * @param object        Pointer to a celestial object data structure that is observed. Catalog
  *                      sources should have coordinates and properties in ICRS. You can use
  *                      `transform_cat()` to convert catalog entries to ICRS as necessary.
- * @param frame         The observer frame, defining the location and time of observation
- * @param sys           The coordinate system in which to return the apparent sky location
+ * @param frame         The observer frame, defining the location and time of observation.
+ * @param sys           The coordinate system in which to return the apparent sky location.
  * @param[out] out      Pointer to the data structure which is populated with the calculated
  *                      apparent location in the designated coordinate system.
  * @return              0 if successful,
@@ -642,7 +727,7 @@ int novas_sky_pos(const object *restrict object, const novas_frame *restrict fra
   if(!object || !frame || !out)
     return novas_error(-1, EINVAL, "NULL argument: object=%p, frame=%p, out=%p", (void *) object, frame, out);
 
-  if(!is_frame_initialized(frame))
+  if(!novas_frame_is_initialized(frame))
     return novas_error(-1, EINVAL, fn, "frame at %p not initialized", frame);
 
   if(frame->accuracy != NOVAS_FULL_ACCURACY && frame->accuracy != NOVAS_REDUCED_ACCURACY)
@@ -704,8 +789,6 @@ int novas_sky_pos(const object *restrict object, const novas_frame *restrict fra
   return 0;
 }
 
-
-
 /**
  * Converts an geometric position in ICRS to an apparent position on sky, by applying appropriate
  * corrections for aberration and gravitational deflection for the observer's frame. Unlike
@@ -738,7 +821,7 @@ int novas_geom_to_app(const novas_frame *restrict frame, const double *restrict
   if(!pos || !frame || !out)
     return novas_error(-1, EINVAL, "NULL argument: pos=%p, frame=%p, out=%p", (void *) pos, frame, out);
 
-  if(!is_frame_initialized(frame))
+  if(!novas_frame_is_initialized(frame))
     return novas_error(-1, EINVAL, fn, "frame at %p not initialized", frame);
 
   if(frame->accuracy != NOVAS_FULL_ACCURACY && frame->accuracy != NOVAS_REDUCED_ACCURACY)
@@ -790,6 +873,7 @@ int novas_geom_to_app(const novas_frame *restrict frame, const double *restrict
  * @sa novas_standard_refraction()
  * @sa novas_optical_refraction()
  * @sa novas_radio_refraction()
+ * @sa novas_wave_refraction()
  *
  * @since 1.1
  * @author Attila Kovacs
@@ -808,7 +892,7 @@ int novas_app_to_hor(const novas_frame *restrict frame, enum novas_reference_sys
     return novas_error(-1, EINVAL, fn, "Both output pointers (az, el) are NULL");
   }
 
-  if(!is_frame_initialized(frame))
+  if(!novas_frame_is_initialized(frame))
     return novas_error(-1, EINVAL, fn, "frame at %p not initialized", frame);
 
   if(frame->observer.where != NOVAS_OBSERVER_ON_EARTH && frame->observer.where != NOVAS_AIRBORNE_OBSERVER) {
@@ -828,6 +912,7 @@ int novas_app_to_hor(const novas_frame *restrict frame, enum novas_reference_sys
       /* fallthrough */
     case NOVAS_TOD:
       spin(15.0 * frame->gst, pos, pos);
+      wobble(time->ijd_tt + time->fjd_tt, WOBBLE_PEF_TO_ITRS, 1e-3 * frame->dx, 1e-3 * frame->dy, pos, pos);
       break;
 
     case NOVAS_ICRS:
@@ -835,7 +920,12 @@ int novas_app_to_hor(const novas_frame *restrict frame, enum novas_reference_sys
       matrix_transform(pos, &frame->gcrs_to_cirs, pos); // @suppress("No break at end of case")
       /* fallthrough */
     case NOVAS_CIRS:
-      spin(frame->era, pos, pos);
+      spin(frame->era, pos, pos); // @suppress("No break at end of case")
+      /* fallthrough */
+    case NOVAS_TIRS:
+      wobble(time->ijd_tt + time->fjd_tt, WOBBLE_TIRS_TO_ITRS, 1e-3 * frame->dx, 1e-3 * frame->dy, pos, pos); // @suppress("No break at end of case")
+      /* fallthrough */
+    case NOVAS_ITRS:
       break;
 
     default:
@@ -881,6 +971,7 @@ int novas_app_to_hor(const novas_frame *restrict frame, enum novas_reference_sys
  * @sa novas_standard_refraction()
  * @sa novas_optical_refraction()
  * @sa novas_radio_refraction()
+ * @sa novas_wave_refraction()
  *
  * @since 1.1
  * @author Attila Kovacs
@@ -898,7 +989,7 @@ int novas_hor_to_app(const novas_frame *restrict frame, double az, double el, Re
     return novas_error(-1, EINVAL, fn, "Both output pointers (ra, dec) are NULL");
   }
 
-  if(!is_frame_initialized(frame))
+  if(!novas_frame_is_initialized(frame))
     return novas_error(-1, EINVAL, fn, "frame at %p not initialized", frame);
 
   if(frame->observer.where != NOVAS_OBSERVER_ON_EARTH && frame->observer.where != NOVAS_AIRBORNE_OBSERVER) {
@@ -913,33 +1004,42 @@ int novas_hor_to_app(const novas_frame *restrict frame, double az, double el, Re
   // az, el to ITRS pos
   hor_to_itrs(&frame->observer.on_surf, az, 90.0 - el, pos);
 
-  // ITRS to TOD or CIRS...
-  spin(cmp_sys(sys, NOVAS_GCRS) < 0 ? -15.0 * frame->gst : -frame->era, pos, pos);
+  if(sys != NOVAS_ITRS) {
+    // ITRS -> PEF / TIRS
+    enum novas_wobble_direction dir = cmp_sys(sys, NOVAS_GCRS) < 0 ? WOBBLE_ITRS_TO_PEF : WOBBLE_ITRS_TO_TIRS;
+    wobble(time->ijd_tt + time->fjd_tt, dir, 1e-3 * frame->dx, 1e-3 * frame->dy, pos, pos);
 
-  // Continue to convert TOD / CIRS to output system....
-  switch(sys) {
-    case NOVAS_TOD:
-      break;
+    if(sys != NOVAS_TIRS) {
+      // TIRS -> TOD or CIRS...
+      spin(cmp_sys(sys, NOVAS_GCRS) < 0 ? -15.0 * frame->gst : -frame->era, pos, pos);
 
-    case NOVAS_MOD:
-      matrix_inv_rotate(pos, &frame->nutation, pos);
-      break;
+      // Continue to convert TOD / CIRS to output system....
+      switch(sys) {
 
-    case NOVAS_J2000:
-      matrix_inv_rotate(pos, &frame->nutation, pos);
-      matrix_inv_rotate(pos, &frame->precession, pos);
-      break;
+        case NOVAS_TOD:
+          break;
 
-    case NOVAS_CIRS:
-      break;
+        case NOVAS_MOD:
+          matrix_inv_rotate(pos, &frame->nutation, pos);
+          break;
 
-    case NOVAS_ICRS:
-    case NOVAS_GCRS:
-      matrix_inv_rotate(pos, &frame->gcrs_to_cirs, pos);
-      break;
+        case NOVAS_J2000:
+          matrix_inv_rotate(pos, &frame->nutation, pos);
+          matrix_inv_rotate(pos, &frame->precession, pos);
+          break;
 
-    default:
-      return novas_error(-1, EINVAL, fn, "invalid coordinate system: %d", sys);
+        case NOVAS_CIRS:
+          break;
+
+        case NOVAS_ICRS:
+        case NOVAS_GCRS:
+          matrix_inv_rotate(pos, &frame->gcrs_to_cirs, pos);
+          break;
+
+        default:
+          return novas_error(-1, EINVAL, fn, "invalid coordinate system: %d", sys);
+      }
+    }
   }
 
   vector2radec(pos, ra, dec);
@@ -976,7 +1076,7 @@ int novas_app_to_geom(const novas_frame *restrict frame, enum novas_reference_sy
   if(!frame || !geom_icrs)
     return novas_error(-1, EINVAL, fn, "NULL argument: frame=%p, geom_icrs=%p", frame, geom_icrs);
 
-  if(!is_frame_initialized(frame))
+  if(!novas_frame_is_initialized(frame))
     return novas_error(-1, EINVAL, fn, "frame at %p not initialized", frame);
 
   if(sys < 0 || sys >= NOVAS_REFERENCE_SYSTEMS)
@@ -989,6 +1089,14 @@ int novas_app_to_geom(const novas_frame *restrict frame, enum novas_reference_sy
 
   // Convert apparent position to ICRS...
   switch(sys) {
+    case NOVAS_ITRS:
+      // ITRS -> TIRS
+      wobble(frame->time.ijd_tt + frame->time.fjd_tt, WOBBLE_ITRS_TO_TIRS, frame->dx, frame->dy, app_pos, app_pos); // @suppress("No break at end of case")
+      /* fallthrough */
+    case NOVAS_TIRS:
+      // TIRS -> CIRS
+      spin(-frame->era, app_pos, app_pos); // @suppress("No break at end of case")
+      /* fallthrough */
     case NOVAS_CIRS:
       matrix_inv_rotate(app_pos, &frame->gcrs_to_cirs, app_pos);
       break;
@@ -1016,20 +1124,20 @@ int novas_app_to_geom(const novas_frame *restrict frame, enum novas_reference_sy
   return 0;
 }
 
-
-static int cat_transform(novas_transform *transform, const novas_matrix *component, int dir) {
+static int add_transform(novas_transform *transform, const novas_matrix *component, int dir) {
   int i;
-  double T[3][3];
+  double M0[3][3];
 
-  memcpy(T, transform->matrix.M, sizeof(T));
-  memset(transform->matrix.M, 0, sizeof(transform->matrix.M));
+  memcpy(M0, transform->matrix.M, sizeof(M0));
 
   for(i = 3; --i >= 0;) {
     int j;
     for(j = 3; --j >= 0;) {
       int k;
+      double x = 0.0;
       for(k = 3; --k >= 0;)
-        transform->matrix.M[i][j] += T[i][k] * (dir < 0 ? component->M[j][k] : component->M[k][j]);
+        x += (dir < 0 ? component->M[k][i] : component->M[i][k]) * M0[k][j];
+      transform->matrix.M[i][j] = x;
     }
   }
 
@@ -1064,7 +1172,7 @@ int novas_make_transform(const novas_frame *frame, enum novas_reference_system f
   if(!frame || !transform)
     return novas_error(-1, EINVAL, fn, "NULL argument: frame=%p, transform=%p", frame, transform);
 
-  if(!is_frame_initialized(frame))
+  if(!novas_frame_is_initialized(frame))
     return novas_error(-1, EINVAL, fn, "frame at %p not initialized", frame);
 
   if(to_system < 0 || to_system >= NOVAS_REFERENCE_SYSTEMS)
@@ -1095,26 +1203,44 @@ int novas_make_transform(const novas_frame *frame, enum novas_reference_system f
 
   if(dir < 0) {
     switch(from_system) {
+      case NOVAS_ITRS: {
+        novas_matrix W;
+        set_wobble_matrix(frame, &W);
+        add_transform(transform, &W, -1);
+        if(to_system == NOVAS_TIRS)
+          return 0;
+      } // @suppress("No break at end of case")
+      /* fallthrough */
+
+      case NOVAS_TIRS: {
+        novas_matrix R;
+        set_spin_matrix(frame->era, &R);
+        add_transform(transform, &R, -1);
+        if(to_system == NOVAS_CIRS)
+          return 0;
+      } // @suppress("No break at end of case")
+      /* fallthrough */
+
       case NOVAS_CIRS:
-        cat_transform(transform, &frame->gcrs_to_cirs, -1);
+        add_transform(transform, &frame->gcrs_to_cirs, -1);
         if(to_system == NOVAS_GCRS)
           return 0; // @suppress("No break at end of case")
         /* fallthrough */
 
       case NOVAS_GCRS:
-        cat_transform(transform, &frame->icrs_to_j2000, 1);
+        add_transform(transform, &frame->icrs_to_j2000, 1);
         if(to_system == NOVAS_J2000)
           return 0; // @suppress("No break at end of case")
         /* fallthrough */
 
       case NOVAS_J2000:
-        cat_transform(transform, &frame->precession, 1);
+        add_transform(transform, &frame->precession, 1);
         if(to_system == NOVAS_MOD)
           return 0; // @suppress("No break at end of case")
         /* fallthrough */
 
       case NOVAS_MOD:
-        cat_transform(transform, &frame->nutation, 1);
+        add_transform(transform, &frame->nutation, 1);
         return 0;
 
       default:
@@ -1125,26 +1251,44 @@ int novas_make_transform(const novas_frame *frame, enum novas_reference_system f
   else {
     switch(from_system) {
       case NOVAS_TOD:
-        cat_transform(transform, &frame->nutation, -1);
+        add_transform(transform, &frame->nutation, -1);
         if(to_system == NOVAS_MOD)
           return 0; // @suppress("No break at end of case")
         /* fallthrough */
 
       case NOVAS_MOD:
-        cat_transform(transform, &frame->precession, -1);
+        add_transform(transform, &frame->precession, -1);
         if(to_system == NOVAS_J2000)
           return 0; // @suppress("No break at end of case")
         /* fallthrough */
 
       case NOVAS_J2000:
-        cat_transform(transform, &frame->icrs_to_j2000, -1);
+        add_transform(transform, &frame->icrs_to_j2000, -1);
         if(to_system == NOVAS_GCRS)
           return 0; // @suppress("No break at end of case")
         /* fallthrough */
 
       case NOVAS_GCRS:
-        cat_transform(transform, &frame->gcrs_to_cirs, 1);
+        add_transform(transform, &frame->gcrs_to_cirs, 1);
+        if(to_system == NOVAS_CIRS)
+          return 0; // @suppress("No break at end of case")
+        /* fallthrough */
+
+      case NOVAS_CIRS: {
+        novas_matrix R;
+        set_spin_matrix(frame->era, &R);
+        add_transform(transform, &R, 1);
+        if(to_system == NOVAS_TIRS)
+          return 0;
+        } // @suppress("No break at end of case")
+        /* fallthrough */
+
+      case NOVAS_TIRS: {
+        novas_matrix W;
+        set_wobble_matrix(frame, &W);
+        add_transform(transform, &W, 1);
         return 0;
+      }
 
       default:
         // nothing to do...
@@ -1182,7 +1326,6 @@ int novas_invert_transform(const novas_transform *transform, novas_transform *in
   return 0;
 }
 
-
 /**
  * Transforms a position or velocity 3-vector from one coordinate reference system to another.
  *
@@ -1257,7 +1400,7 @@ double novas_frame_lst(const novas_frame *restrict frame) {
     return NAN;
   }
 
-  if(!is_frame_initialized(frame)) {
+  if(!novas_frame_is_initialized(frame)) {
     novas_error(0, EINVAL, fn, "input frame is not initialized");
     return NAN;
   }
@@ -1523,7 +1666,7 @@ double novas_solar_illum(const object *restrict source, const novas_frame *restr
     return NAN;
   }
 
-  if(!is_frame_initialized(frame)) {
+  if(!novas_frame_is_initialized(frame)) {
     novas_error(0, EINVAL, fn, "input frame is not initialized");
     return NAN;
   }
@@ -1692,7 +1835,7 @@ int novas_equ_track(const object *restrict source, const novas_frame *restrict f
   if(!frame)
     return novas_error(-1, EINVAL, fn, "input frame is NULL");
 
-  if(!is_frame_initialized(frame))
+  if(!novas_frame_is_initialized(frame))
     return novas_error(-1, EINVAL, fn, "input frame is not initialized");
 
   if(!track)
@@ -1782,7 +1925,7 @@ int novas_hor_track(const object *restrict source, const novas_frame *restrict f
   if(!frame)
     return novas_error(-1, EINVAL, fn, "input frame is NULL");
 
-  if(!is_frame_initialized(frame))
+  if(!novas_frame_is_initialized(frame))
     return novas_error(-1, EINVAL, fn, "input frame is not initialized");
 
   if(frame->observer.where != NOVAS_OBSERVER_ON_EARTH && frame->observer.where != NOVAS_AIRBORNE_OBSERVER)
