@@ -161,15 +161,11 @@ short ter2cel(double jd_ut1_high, double jd_ut1_low, double ut1_to_tt, enum nova
   // For these calculations we can assume TDB = TT (< 2 ms difference)
   jd_tdb = jd_tt;
 
-  // Apply polar motion
-  if(xp || yp)
-    wobble(jd_tt, WOBBLE_ITRS_TO_PEF, xp, yp, in, out);
-  else if(out != in)
-    memcpy(out, in, XYZ_VECTOR_SIZE);
 
   switch(erot) {
-    case (EROT_ERA): {
+    case EROT_ERA:
       // 'CIO-TIO-THETA' method. See second reference, eq. (3) and (4).
+      wobble(jd_tt, WOBBLE_ITRS_TO_TIRS, xp, yp, in, out);
 
       // Compute and apply the Earth rotation angle, 'theta', transforming the
       // vector to the celestial intermediate system.
@@ -179,10 +175,15 @@ short ter2cel(double jd_ut1_high, double jd_ut1_low, double ut1_to_tt, enum nova
         prop_error(fn, cirs_to_gcrs(jd_tdb, accuracy, out, out), 10);
 
       break;
-    }
-    case (EROT_GST): {
+
+    case EROT_GST: {
       double gast;
 
+      if(xp || yp)
+        wobble(jd_tt, WOBBLE_ITRS_TO_PEF, xp, yp, in, out);
+      else
+        memcpy(out, in, XYZ_VECTOR_SIZE);
+
       sidereal_time(jd_ut1_high, jd_ut1_low, ut1_to_tt, NOVAS_TRUE_EQUINOX, EROT_GST, accuracy, &gast);
       spin(-15.0 * gast, out, out);
 
@@ -287,7 +288,10 @@ short cel2ter(double jd_ut1_high, double jd_ut1_low, double ut1_to_tt, enum nova
       // Compute and apply the Earth rotation angle, 'theta', transforming the
       // vector to the terrestrial intermediate system.
       spin(era(jd_ut1_high, jd_ut1_low), out, out);
-      break;
+
+      // Apply polar motion, transforming the vector to the ITRS.
+      wobble(jd_tt, WOBBLE_TIRS_TO_ITRS, xp, yp, out, out);
+      return 0;
 
     case EROT_GST: {
       double gast;
@@ -303,18 +307,15 @@ short cel2ter(double jd_ut1_high, double jd_ut1_low, double ut1_to_tt, enum nova
       // Apply Earth rotation.
       sidereal_time(jd_ut1_high, jd_ut1_low, ut1_to_tt, NOVAS_TRUE_EQUINOX, EROT_GST, accuracy, &gast);
       spin(15.0 * gast, out, out);
-      break;
-    }
 
-    default:
-      return novas_error(2, EINVAL, fn, "invalid Earth rotation measure type: %d", erot);
+      // Apply polar motion, transforming the vector to the ITRS.
+      if(xp || yp)
+        wobble(jd_tt, WOBBLE_PEF_TO_ITRS, xp, yp, out, out);
+      return 0;
+    }
   }
 
-  // Apply polar motion, transforming the vector to the ITRS.
-  if(xp || yp)
-    wobble(jd_tt, WOBBLE_PEF_TO_ITRS, xp, yp, out, out);
-
-  return 0;
+  return novas_error(2, EINVAL, fn, "invalid Earth rotation measure type: %d", erot);
 }
 
 
@@ -1028,287 +1029,3 @@ int tod_to_cirs(double jd_tt, enum novas_accuracy accuracy, const double *in, do
 
   return 0;
 }
-
-/**
- * Precesses equatorial rectangular coordinates from one epoch to another. Unlike the original
- * NOVAS routine, this routine works for any pairing of the time arguments.
- *
- * This function calculates precession for the old (pre IAU 2000) methodology. Its main use
- * for NOVAS users is to allow converting older catalog coordinates e.g. to J2000 coordinates,
- * which then can be converted to the now standard ICRS system via frame_tie().
- *
- * NOTE:
- * <ol>
- * <li>Unlike the original NOVAS C 3.1 version, this one does not require that one
- *     of the time arguments must be J2000. You can precess from any date to
- *     any other date, and the intermediate epoch of J2000 will be handled internally
- *     as needed.</li>
- *
- * <li>This function caches the results of the last calculation in case it may be re-used at
- *     no extra computational cost for the next call.</li>
- * </ol>
- *
- * REFERENCES:
- * <ol>
- * <li>Explanatory Supplement To The Astronomical Almanac, pp. 103-104.</li>
- * <li>Capitaine, N. et al. (2003), Astronomy And Astrophysics 412, pp. 567-586.</li>
- * <li>Hilton, J. L. et al. (2006), IAU WG report, Celest. Mech., 94, pp. 351-367.</li>
- * </ol>
- *
- * @param jd_tdb_in   [day] Barycentric Dynamic Time (TDB) based Julian date of the input
- *                    epoch
- * @param in          Position 3-vector, geocentric equatorial rectangular coordinates,
- *                    referred to mean dynamical equator and equinox of the initial epoch.
- * @param jd_tdb_out  [day] Barycentric Dynamic Time (TDB) based Julian date of the output
- *                    epoch
- * @param[out] out    Position 3-vector, geocentric equatorial rectangular coordinates,
- *                    referred to mean dynamical equator and equinox of the final epoch.
- *                    It can be the same vector as the input.
- * @return            0 if successful, or -1 if either of the position vectors is NULL.
- *
- * @sa nutation()
- * @sa frame_tie()
- * @sa novas_epoch()
- * @sa tt2tdb()
- * @sa cio_basis()
- * @sa NOVAS_TOD
- * @sa NOVAS_JD_J2000
- * @sa NOVAS_JD_B1950
- * @sa NOVAS_JD_B1900
- */
-short precession(double jd_tdb_in, const double *in, double jd_tdb_out, double *out) {
-  static THREAD_LOCAL double djd_last[2] = { NAN, NAN };
-  static THREAD_LOCAL double xx[2], yx[2], zx[2], xy[2], yy[2], zy[2], xz[2], yz[2], zz[2];
-
-  double t;
-  int i = 0;
-
-  if(!in || !out)
-    return novas_error(-1, EINVAL, "precession", "NULL input or output 3-vector: in=%p, out=%p", in, out);
-
-  if(jd_tdb_in == jd_tdb_out) {
-    if(out != in)
-      memcpy(out, in, XYZ_VECTOR_SIZE);
-    return 0;
-  }
-
-  // Check to be sure that either 'jd_tdb1' or 'jd_tdb2' is equal to JD_J2000.
-  if(!novas_time_equals(jd_tdb_in, JD_J2000) && !novas_time_equals(jd_tdb_out, JD_J2000)) {
-    // Do the precession in two steps...
-    precession(jd_tdb_in, in, JD_J2000, out);
-    precession(JD_J2000, out, jd_tdb_out, out);
-    return 0;
-  }
-
-  // 't' is time in TDB centuries between the two epochs.
-  t = (jd_tdb_out - jd_tdb_in);
-  if(jd_tdb_out == JD_J2000) {
-    t = -t;
-    i = 1;
-  }
-
-  if(!novas_time_equals(t, djd_last[i])) {
-    double psia, omegaa, chia, sa, ca, sb, cb, sc, cc, sd, cd, t1, t2;
-    double eps0 = 84381.406;
-
-    djd_last[i] = t;
-
-    // Now change t to Julian centuries
-    t /= JULIAN_CENTURY_DAYS;
-
-    // Numerical coefficients of psi_a, omega_a, and chi_a, along with
-    // epsilon_0, the obliquity at J2000.0, are 4-angle formulation from
-    // Capitaine et al. (2003), eqs. (4), (37), & (39).
-    psia = ((((-0.0000000951 * t + 0.000132851) * t - 0.00114045) * t - 1.0790069) * t + 5038.481507) * t;
-    omegaa = ((((+0.0000003337 * t - 0.000000467) * t - 0.00772503) * t + 0.0512623) * t - 0.025754) * t + eps0;
-    chia = ((((-0.0000000560 * t + 0.000170663) * t - 0.00121197) * t - 2.3814292) * t + 10.556403) * t;
-
-    eps0 *= ARCSEC;
-    psia *= ARCSEC;
-    omegaa *= ARCSEC;
-    chia *= ARCSEC;
-
-    sa = sin(eps0);
-    ca = cos(eps0);
-    sb = sin(-psia);
-    cb = cos(-psia);
-    sc = sin(-omegaa);
-    cc = cos(-omegaa);
-    sd = sin(chia);
-    cd = cos(chia);
-
-    // Compute elements of precession rotation matrix equivalent to
-    // R3(chi_a) R1(-omega_a) R3(-psi_a) R1(epsilon_0).
-    t1 = cd * sb + sd * cc * cb;
-    t2 = sd * sc;
-    xx[i] = cd * cb - sb * sd * cc;
-    yx[i] = ca * t1 - sa * t2;
-    zx[i] = sa * t1 + ca * t2;
-
-    t1 = cd * cc * cb - sd * sb;
-    t2 = cd * sc;
-    xy[i] = -sd * cb - sb * cd * cc;
-    yy[i] = ca * t1 - sa * t2;
-    zy[i] = sa * t1 + ca * t2;
-
-    xz[i] = sb * sc;
-    yz[i] = -sc * cb * ca - sa * cc;
-    zz[i] = -sc * cb * sa + cc * ca;
-  }
-
-  if(jd_tdb_out == JD_J2000) {
-    const double x = in[0], y = in[1], z = in[2];
-    // Perform rotation from epoch to J2000.0.
-    out[0] = xx[1] * x + xy[1] * y + xz[1] * z;
-    out[1] = yx[1] * x + yy[1] * y + yz[1] * z;
-    out[2] = zx[1] * x + zy[1] * y + zz[1] * z;
-  }
-  else {
-    const double x = in[0], y = in[1], z = in[2];
-    // Perform rotation from J2000.0 to epoch.
-    out[0] = xx[0] * x + yx[0] * y + zx[0] * z;
-    out[1] = xy[0] * x + yy[0] * y + zy[0] * z;
-    out[2] = xz[0] * x + yz[0] * y + zz[0] * z;
-  }
-
-  return 0;
-}
-
-/**
- * Nutates equatorial rectangular coordinates from mean equator and equinox of epoch to true
- * equator and equinox of epoch. Inverse transformation may be applied by setting flag
- * 'direction'.
- *
- * This is the old (pre IAU 2006) method of nutation calculation. If you follow the now
- * standard IAU 2000/2006 methodology you will want to use nutation_angles() instead.
- *
- * REFERENCES:
- * <ol>
- * <li>Explanatory Supplement To The Astronomical Almanac, pp. 114-115.</li>
- * </ol>
- *
- *
- * @param jd_tdb      [day] Barycentric Dynamic Time (TDB) based Julian date
- * @param direction   NUTATE_MEAN_TO_TRUE (0) or NUTATE_TRUE_TO_MEAN (-1; or non-zero)
- * @param accuracy    NOVAS_FULL_ACCURACY (0) or NOVAS_REDUCED_ACCURACY (1)
- * @param in          Position 3-vector, geocentric equatorial rectangular coordinates,
- *                    referred to mean equator and equinox of epoch.
- * @param[out] out    Position vector, geocentric equatorial rectangular coordinates,
- *                    referred to true equator and equinox of epoch. It can be the same
- *                    as the input position.
- *
- * @return            0 if successful, or -1 if one of the vector arguments is NULL.
- *
- * @sa nutation_angles()
- * @sa tt2tdb()
- * @sa NOVAS_TOD
- */
-int nutation(double jd_tdb, enum novas_nutation_direction direction, enum novas_accuracy accuracy, const double *in, double *out) {
-  double oblm, oblt, psi;
-  double cm, sm, ct, st, cp, sp;
-  double xx, yx, zx, xy, yy, zy, xz, yz, zz;
-
-  if(!in || !out)
-    return novas_error(-1, EINVAL, "nutation", "NULL input or output 3-vector: in=%p, out=%p", in, out);
-
-  // Call 'e_tilt' to get the obliquity and nutation angles.
-  prop_error("nutation", e_tilt(jd_tdb, accuracy, &oblm, &oblt, NULL, &psi, NULL), 0);
-
-  oblm *= DEGREE;
-  oblt *= DEGREE;
-  psi *= ARCSEC;
-
-  cm = cos(oblm);
-  sm = sin(oblm);
-  ct = cos(oblt);
-  st = sin(oblt);
-  cp = cos(psi);
-  sp = sin(psi);
-
-  // Nutation rotation matrix follows.
-  xx = cp;
-  yx = -sp * cm;
-  zx = -sp * sm;
-  xy = sp * ct;
-  yy = cp * cm * ct + sm * st;
-  zy = cp * sm * ct - cm * st;
-  xz = sp * st;
-  yz = cp * cm * st - sm * ct;
-  zz = cp * sm * st + cm * ct;
-
-  if(direction == NUTATE_MEAN_TO_TRUE) {
-    const double x = in[0], y = in[1], z = in[2];
-    // Perform rotation.
-    out[0] = xx * x + yx * y + zx * z;
-    out[1] = xy * x + yy * y + zy * z;
-    out[2] = xz * x + yz * y + zz * z;
-  }
-  else {
-    const double x = in[0], y = in[1], z = in[2];
-    // Perform inverse rotation.
-    out[0] = xx * x + xy * y + xz * z;
-    out[1] = yx * x + yy * y + yz * z;
-    out[2] = zx * x + zy * y + zz * z;
-  }
-
-  return 0;
-}
-
-/**
- * Returns the values for nutation in longitude and nutation in obliquity for a given TDB
- * Julian date.  The nutation model selected depends upon the input value of 'accuracy'. See
- * notes below for important details.
- *
- * This function selects the nutation model depending first upon the input value of 'accuracy'.
- * If 'accuracy' is NOVAS_FULL_ACCURACY (0), the IAU 2000A nutation model is used. Otherwise
- * the model set by set_nutation_lp_provider() is used, or else the default nu2000k().
- *
- * See the prologs of the nutation functions in file 'nutation.c' for details concerning the
- * models.
- *
- * REFERENCES:
- * <ol>
- * <li>Kaplan, G. (2005), US Naval Observatory Circular 179.</li>
- * </ol>
- *
- * @param t           [cy] TDB time in Julian centuries since J2000.0
- * @param accuracy    NOVAS_FULL_ACCURACY (0) or NOVAS_REDUCED_ACCURACY (1)
- * @param[out] dpsi   [arcsec] Nutation in longitude in arcseconds.
- * @param[out] deps   [arcsec] Nutation in obliquity in arcseconds.
- *
- * @return            0 if successful, or -1 if the output pointer arguments are NULL
- *
- * @sa set_nutation_lp_provider()
- * @sa nutation()
- * @sa iau2000b()
- * @sa nu2000k()
- * @sa cio_basis()
- * @sa NOVAS_CIRS
- * @sa NOVAS_JD_J2000
- */
-int nutation_angles(double t, enum novas_accuracy accuracy, double *restrict dpsi, double *restrict deps) {
-  if(!dpsi || !deps) {
-    if(dpsi)
-      *dpsi = NAN;
-    if(deps)
-      *deps = NAN;
-
-    return novas_error(-1, EINVAL, "nutation_angles", "NULL output pointer: dspi=%p, deps=%p", dpsi, deps);
-  }
-
-  if(accuracy == NOVAS_FULL_ACCURACY) {
-    // High accuracy mode -- use IAU 2000A.
-    iau2000a(JD_J2000, t * JULIAN_CENTURY_DAYS, dpsi, deps);
-  }
-  else {
-    // Low accuracy mode -- model depends upon value of 'low_acc_choice'.
-    novas_nutation_provider nutate_call = get_nutation_lp_provider();
-    nutate_call(JD_J2000, t * JULIAN_CENTURY_DAYS, dpsi, deps);
-  }
-
-  // Convert output to arcseconds.
-  *dpsi /= ARCSEC;
-  *deps /= ARCSEC;
-
-  return 0;
-}
