@@ -47,8 +47,16 @@ static int check_nan(const char *func, double value) {
 }
 
 static int test_make_on_surface() {
+  int n = 0;
+
+  on_surface loc = ON_SURFACE_INIT;
+
   if(check("make_on_surface", -1, make_on_surface(0.0, 0.0, 0.0, 0.0, 0.0, NULL))) return 1;
-  return 0;
+  if(check("make_on_surface:temp:lo", -1, make_on_surface(0.0, 0.0, 0.0, -120.1, 0.0, &loc))) n++;
+  if(check("make_on_surface:temp:hi", -1, make_on_surface(0.0, 0.0, 0.0, 70.1, 0.0, &loc))) n++;
+  if(check("make_on_surface:pressure:lo", -1, make_on_surface(0.0, 0.0, 0.0, 0.0, -0.1, &loc))) n++;
+  if(check("make_on_surface:pressure:hi", -1, make_on_surface(0.0, 0.0, 0.0, 0.0, 1200.1, &loc))) n++;
+  return n;
 }
 
 static int test_make_in_space() {
@@ -185,24 +193,45 @@ static int test_z2v() {
 }
 
 static int test_refract() {
-  on_surface o = ON_SURFACE_INIT;
+  on_surface loc = ON_SURFACE_INIT, loc0 = ON_SURFACE_INIT;
   int n = 0;
 
   novas_debug(NOVAS_DEBUG_ON);
   fprintf(stderr, ">>> Expecting error message...\n");
+
   errno = 0;
   double r = refract(NULL, NOVAS_STANDARD_ATMOSPHERE, 30.0);
   if(check("refract:loc", 1, r == 0.0 && errno == EINVAL)) n++;
   novas_debug(NOVAS_DEBUG_OFF);
 
   errno = 0;
-  r = refract(&o, -1, 30.0);
-  if(check("refract:model", 1, r == 0.0 && errno == EINVAL)) n++;
+  r = refract(&loc, -1, 30.0);
+  if(check("refract:model:-1:ret", 1, r == 0.0)) n++;
+  if(check("refract:model:-1:errno", EINVAL, errno)) n++;
 
   errno = 0;
-  r = refract(&o, NOVAS_STANDARD_ATMOSPHERE, 91.01);
+  r = refract(&loc, NOVAS_REFRACTION_MODELS, 30.0);
+  if(check("refract:model:hi:ret", 1, r == 0.0)) n++;
+  if(check("refract:model:hi:errno", EINVAL, errno)) n++;
+
+  errno = 0;
+  r = refract(&loc, NOVAS_STANDARD_ATMOSPHERE, 91.01);
   if(check("refract:zd", 1, r == 0.0)) n++;
 
+  loc = loc0;
+  loc.temperature = -150.1;
+  if(check_nan("refract:loc:temperature:lo", refract(&loc, NOVAS_WEATHER_AT_LOCATION, 30.0))) n++;
+
+  loc.temperature = 100.1;
+  if(check_nan("refract:loc:temperature:hi", refract(&loc, NOVAS_WEATHER_AT_LOCATION, 30.0))) n++;
+
+  loc = loc0;
+  loc.pressure = -0.1;
+  if(check_nan("refract:loc:pressure:lo", refract(&loc, NOVAS_WEATHER_AT_LOCATION, 30.0))) n++;
+
+  loc.pressure = 2000.1;
+  if(check_nan("refract:loc:pressure:hi", refract(&loc, NOVAS_WEATHER_AT_LOCATION, 30.0))) n++;
+
   return n;
 }
 
@@ -211,6 +240,10 @@ static int test_refract_astro() {
   on_surface surf = ON_SURFACE_INIT;
   int n = 0;
 
+  if(check_nan("refract_astro:loc", refract_astro(NULL, NOVAS_STANDARD_ATMOSPHERE, 30.0))) n++;
+  if(check_nan("refract_astro:model:-1", refract_astro(&surf, -1, 30.0))) n++;
+  if(check_nan("refract_astro:model:hi", refract_astro(&surf, NOVAS_REFRACTION_MODELS, 30.0))) n++;
+
   novas_inv_max_iter = 0;
   if(check_nan("refract_astro:converge", refract_astro(&surf, NOVAS_STANDARD_ATMOSPHERE, 85.0))) n++;
   else if(check("refract_astro:converge:errno", ECANCELED, errno)) n++;
@@ -225,6 +258,8 @@ static int test_inv_refract() {
   on_surface surf = ON_SURFACE_INIT;
   int n = 0;
 
+  if(check_nan("inv_refract:loc", novas_inv_refract(novas_optical_refraction, NOVAS_JD_J2000, NULL, NOVAS_REFRACT_OBSERVED, 5.0))) n++;;
+
   novas_inv_max_iter = 0;
   if(check_nan("inv_refract:converge", novas_inv_refract(novas_optical_refraction, NOVAS_JD_J2000, &surf, NOVAS_REFRACT_OBSERVED, 5.0))) n++;
   else if(check("inv_refract:converge:errno", ECANCELED, errno)) n++;
@@ -233,6 +268,89 @@ static int test_inv_refract() {
   return n;
 }
 
+static int test_radio_refraction() {
+  int n = 0;
+
+  on_surface loc = ON_SURFACE_INIT, loc0 = ON_SURFACE_INIT;
+
+  if(check_nan("radio_refraction:loc:null", novas_radio_refraction(NOVAS_JD_J2000, NULL, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+  if(check_nan("radio_refraction:loc:type:1", novas_radio_refraction(NOVAS_JD_J2000, &loc, 1, 30.0))) n++;
+  if(check_nan("radio_refraction:el:lo", novas_radio_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, -1.01))) n++;
+  if(check_nan("radio_refraction:el:hi", novas_radio_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 90.01))) n++;
+
+  loc.temperature = -150.1;
+  if(check_nan("radio_refraction:temperature:lo", novas_radio_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  loc.temperature = 100.1;
+  if(check_nan("radio_refraction:temperature:hi", novas_radio_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  loc = loc0;
+  loc.pressure = -0.1;
+  if(check_nan("radio_refraction:pressure:lo", novas_radio_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  loc.pressure = 2000.1;
+  if(check_nan("radio_refraction:pressure:hi", novas_radio_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  loc = loc0;
+  loc.humidity = -0.1;
+  if(check_nan("radio_refraction:humidity:lo", novas_radio_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  loc.humidity = 100.1;
+  if(check_nan("radio_refraction:humidity:hi", novas_radio_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  return n;
+}
+
+static int test_wave_refraction() {
+  int n = 0;
+
+  on_surface loc = ON_SURFACE_INIT, loc0 = ON_SURFACE_INIT;
+
+  if(check_nan("wave_refraction:loc:null", novas_wave_refraction(NOVAS_JD_J2000, NULL, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+  if(check_nan("wave_refraction:loc:type:1", novas_wave_refraction(NOVAS_JD_J2000, &loc, 1, 30.0))) n++;
+  if(check_nan("wave_refraction:el:lo", novas_wave_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_OBSERVED, 0.0))) n++;
+  if(check_nan("wave_refraction:el:hi", novas_wave_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_OBSERVED, 90.01))) n++;
+
+  loc.temperature = -150.1;
+  if(check_nan("wave_refraction:temperature:lo", novas_wave_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  loc.temperature = 200.1;
+  if(check_nan("wave_refraction:temperature:hi", novas_wave_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  loc = loc0;
+  loc.pressure = -0.1;
+  if(check_nan("wave_refraction:pressure:lo", novas_wave_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  loc.pressure = 10000.1;
+  if(check_nan("wave_refraction:pressure:hi", novas_wave_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  loc = loc0;
+  loc.humidity = -0.1;
+  if(check_nan("wave_refraction:humidity:lo", novas_wave_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  loc.humidity = 100.1;
+  if(check_nan("wave_refraction:humidity:hi", novas_wave_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  loc = loc0;
+  novas_refract_wavelength(0.099);
+  if(check_nan("wave_refraction:wl:lo", novas_wave_refraction(NOVAS_JD_J2000, &loc, NOVAS_REFRACT_ASTROMETRIC, 30.0))) n++;
+
+  novas_refract_wavelength(NOVAS_DEFAULT_WAVELENGTH);
+
+  return n;
+}
+
+static int test_refract_wavelength() {
+  int n = 0;
+
+  if(check("refract:wavelength:0", -1, novas_refract_wavelength(0.0))) n++;
+  if(check("refract:wavelength:neg", -1, novas_refract_wavelength(-0.1))) n++;
+  if(check("refract:wavelength:nan", -1, novas_refract_wavelength(NAN))) n++;
+
+  return n++;
+}
+
+
 static int test_limb_angle() {
   double pos[3] = { 0.01 }, pn[3] = { -0.01 }, pz[3] = {0.0}, a, b;
   int n = 0;
@@ -430,10 +548,12 @@ static int test_place() {
   int n = 0;
 
   if(check("place:object", -1, place(0.0, NULL, &loc, 0.0, NOVAS_GCRS, NOVAS_FULL_ACCURACY, &pos))) n++;
-  if(check("place:sys:lo", 1, place(0.0, &o, &loc, 0.0, -1, NOVAS_FULL_ACCURACY, &pos))) n++;
+  if(check("place:sys:-1", 1, place(0.0, &o, &loc, 0.0, -1, NOVAS_FULL_ACCURACY, &pos))) n++;
+  if(check("place:sys:itrs", 1, place(0.0, &o, &loc, 0.0, NOVAS_ITRS, NOVAS_FULL_ACCURACY, &pos))) n++;
   if(check("place:sys:hi", 1, place(0.0, &o, &loc, 0.0, NOVAS_REFERENCE_SYSTEMS, NOVAS_FULL_ACCURACY, &pos))) n++;
   if(check("place:accuracy", 2, place(0.0, &o, &loc, 0.0, NOVAS_GCRS, -1, &pos))) n++;
 
+
   return n;
 }
 
@@ -625,6 +745,8 @@ static int test_wobble() {
   double p[3] = {1.0};
   int n = 0;
 
+  if(check("wobble:dir:-1", 0, wobble(NOVAS_JD_J2000, -1, 0.0, 0.0, p, p))) n++;
+  if(check("wobble:dir:hi", -1, wobble(NOVAS_JD_J2000, NOVAS_WOBBLE_DIRECTIONS, 0.0, 0.0, p, p))) n++;
   if(check("wobble:in", -1, wobble(0.0, WOBBLE_ITRS_TO_PEF, 0.0, 0.0, NULL, p))) n++;
   if(check("wobble:out", -1, wobble(0.0, WOBBLE_ITRS_TO_PEF, 0.0, 0.0, p, NULL))) n++;
 
@@ -1023,7 +1145,7 @@ static int test_grav_undo_planets() {
   if(check("grav_undo_planets:pos_app", -1, grav_undo_planets(NULL, po, &planets, out))) n++;
   if(check("grav_undo_planets:pos_obs", -1, grav_undo_planets(p, NULL, &planets, out))) n++;
   if(check("grav_undo_planets:planets", -1, grav_undo_planets(p, po, NULL, out))) n++;
-    if(check("grav_undo_planets:pos_src", -1, grav_undo_planets(p, po, &planets, NULL))) n++;
+  if(check("grav_undo_planets:pos_src", -1, grav_undo_planets(p, po, &planets, NULL))) n++;
 
   planets.mask = 1 << NOVAS_SUN;
   novas_inv_max_iter = 0;
@@ -1594,11 +1716,17 @@ static int test_orbit_posvel() {
   orbit.system.type = -1;
   if(check("set_obsys_pole:orbit:type:-1", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
 
+  orbit.system.type = NOVAS_TIRS;
+  if(check("set_obsys_pole:orbit:type:tirs", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
+
+  orbit.system.type = NOVAS_ITRS;
+  if(check("set_obsys_pole:orbit:type:itrs", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
+
   orbit.system.type = NOVAS_REFERENCE_SYSTEMS;
   if(check("set_obsys_pole:orbit:type:hi", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
 
   orbit.system.plane = NOVAS_EQUATORIAL_PLANE;
-  orbit.system.type = NOVAS_REFERENCE_SYSTEMS;
+  orbit.system.type = -1;
   if(check("set_obsys_pole:orbit:type:-1:eq", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
 
   orbit.system.type = NOVAS_GCRS;
@@ -1611,7 +1739,12 @@ static int test_orbit_posvel() {
 static int test_set_obsys_pole() {
   int n = 0;
 
-  if(check("set_obsys_pole:orbit", -1, novas_set_orbsys_pole(NOVAS_GCRS, 0.0, 0.0, NULL))) n++;
+  if(check("set_obsys_pole:orbit:null", -1, novas_set_orbsys_pole(NOVAS_GCRS, 0.0, 0.0, NULL))) n++;
+
+  if(check("set_obsys_pole:orbit:-1", -1, novas_set_orbsys_pole(-1, 0.0, 0.0, NULL))) n++;
+  if(check("set_obsys_pole:orbit:tirs", -1, novas_set_orbsys_pole(NOVAS_TIRS, 0.0, 0.0, NULL))) n++;
+  if(check("set_obsys_pole:orbit:itrs", -1, novas_set_orbsys_pole(NOVAS_ITRS, 0.0, 0.0, NULL))) n++;
+  if(check("set_obsys_pole:orbit:hi", -1, novas_set_orbsys_pole(NOVAS_REFERENCE_SYSTEMS, 0.0, 0.0, NULL))) n++;
 
   return n;
 }
@@ -1666,6 +1799,9 @@ static int test_parse_degrees() {
 
   if(check_nan("parse_degrees:null", novas_parse_degrees(NULL, &tail))) n++;
   if(check_nan("parse_degrees:blah", novas_parse_degrees("blah", &tail))) n++;
+  if(check_nan("parse_degrees:East+space", novas_parse_degrees("East ", &tail))) n++;
+  if(check_nan("parse_degrees:East+blah", novas_parse_degrees("East blah", &tail))) n++;
+  if(check_nan("parse_degrees:East..0", novas_parse_degrees("East ..0", &tail))) n++;
 
   return n;
 }
@@ -1887,6 +2023,26 @@ static int test_julian_date() {
   return n;
 }
 
+static int test_jd_from_date() {
+  int n = 0;
+
+  if(check("jd_from_date:calendar:-2", -1, novas_jd_from_date(-2, 2000, 1, 1, 0.0))) n++;
+  if(check("jd_from_date:calendar:2", -1, novas_jd_from_date(2, 2000, 1, 1, 0.0))) n++;
+
+  return n;
+}
+
+static int test_jd_to_date() {
+  int n = 0;
+  int y, m, d;
+  double h;
+
+  if(check("jd_to_date:calendar:-2", -1, novas_jd_to_date(NOVAS_JD_J2000, -2, &y, &m, &d, &h))) n++;
+  if(check("jd_to_date:calendar:2", -1, novas_jd_to_date(NOVAS_JD_J2000, 2, &y, &m, &d, &h))) n++;
+
+  return n;
+}
+
 static int test_parse_date() {
   int n = 0;
   char *tail = NULL;
@@ -2073,6 +2229,87 @@ static int test_print_dms() {
   return n;
 }
 
+static int test_time_lst() {
+  int n = 0;
+  novas_timespec t = NOVAS_TIMESPEC_INIT;
+
+  if(check_nan("time_lst:time:null", novas_time_lst(NULL, 0.0, NOVAS_FULL_ACCURACY))) n++;
+  if(check_nan("time_lst:acc:-1", novas_time_lst(&t, 0.0, -1))) n++;
+
+  return n;
+}
+
+static int test_novas_make_planet_orbit() {
+  int n = 0;
+
+  novas_orbital orbit = NOVAS_ORBIT_INIT;
+
+  if(check("novas_make_planet_orbit:sun", -1, novas_make_planet_orbit(NOVAS_SUN, NOVAS_JD_J2000, &orbit))) n++;
+  if(check("novas_make_planet_orbit:ssb", -1, novas_make_planet_orbit(NOVAS_SSB, NOVAS_JD_J2000, &orbit))) n++;
+  if(check("novas_make_planet_orbit:jd:lo", -1, novas_make_planet_orbit(NOVAS_EARTH, 0.0, &orbit))) n++;
+  if(check("novas_make_planet_orbit:jd:hi", -1, novas_make_planet_orbit(NOVAS_EARTH, 2 * NOVAS_JD_J2000, &orbit))) n++;
+  if(check("novas_make_planet_orbit:orbit:null", -1, novas_make_planet_orbit(NOVAS_EARTH, NOVAS_JD_J2000, NULL))) n++;
+
+  return n;
+}
+
+static int test_novas_make_moon_orbit() {
+  if(check("novas_make_moon_orbit:orbit:null", -1, novas_make_moon_orbit(NOVAS_JD_J2000, NULL))) return 1;
+  return 0;
+}
+
+static int test_approx_heliocentric() {
+  double p[3] = {0.0}, v[3] = {0.0};
+
+  if(check("approx_heliocentric:pos=vel", -1, novas_approx_heliocentric(NOVAS_EMB, NOVAS_JD_J2000, NULL, NULL))) return 1;
+  if(check("approx_heliocentric:time:lo", -1, novas_approx_heliocentric(NOVAS_EMB, NOVAS_JD_J2000 - 31.0 * JULIAN_CENTURY_DAYS, p, v))) return 1;
+  if(check("approx_heliocentric:time:hi", -1, novas_approx_heliocentric(NOVAS_EMB, NOVAS_JD_J2000 + 31.0 * JULIAN_CENTURY_DAYS, p, v))) return 1;
+
+  return 0;
+}
+
+static int test_approx_sky_pos() {
+  novas_timespec ts = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+  sky_pos out = SKY_POS_INIT;
+  int n = 0;
+
+  if(check("approx_sky_pos:frame:null", -1, novas_approx_sky_pos(NOVAS_EMB, NULL, NOVAS_ICRS, &out))) n++;
+  if(check("approx_sky_pos:frame:init", -1, novas_approx_sky_pos(NOVAS_EMB, &frame, NOVAS_ICRS, &out))) n++;
+
+  novas_set_time(NOVAS_TT, NOVAS_JD_J2000, 32, 0.0, &ts);
+  make_observer_at_geocenter(&obs);
+  novas_make_frame(NOVAS_REDUCED_ACCURACY, &obs, &ts, 0.0, 0.0, &frame);
+
+  if(check("approx_sky_pos:out:null", -1, novas_approx_sky_pos(NOVAS_EMB, &frame, NOVAS_ICRS, NULL))) n++;
+
+  return n;
+}
+
+static int test_moon_phase() {
+  int n = 0;
+
+  int saved = novas_inv_max_iter;
+  novas_inv_max_iter = 0;
+  if(check_nan("moon_phase:conv", novas_moon_phase(NOVAS_JD_J2000))) n++;
+  novas_inv_max_iter = saved;
+  return n;
+}
+
+static int test_next_moon_phase() {
+  int n = 0;
+
+  if(check_nan("next_moon_phase:time:lo", novas_next_moon_phase(0.0, NOVAS_JD_J2000 - 31.0 * JULIAN_CENTURY_DAYS))) n++;
+  if(check_nan("next_moon_phase:time:hi", novas_next_moon_phase(0.0, NOVAS_JD_J2000 + 31.0 * JULIAN_CENTURY_DAYS))) n++;
+
+  int saved = novas_inv_max_iter;
+  novas_inv_max_iter = 0;
+  if(check_nan("next_moon_phase:conv", novas_next_moon_phase(0.0, NOVAS_JD_J2000))) n++;
+  novas_inv_max_iter = saved;
+  return n;
+}
+
 int main() {
   int n = 0;
 
@@ -2096,6 +2333,9 @@ int main() {
   if(test_refract()) n++;
   if(test_refract_astro()) n++;
   if(test_inv_refract()) n++;
+  if(test_radio_refraction()) n++;
+  if(test_wave_refraction()) n++;
+  if(test_refract_wavelength()) n++;
   if(test_limb_angle()) n++;
 
   if(test_ephemeris()) n++;
@@ -2228,6 +2468,8 @@ int main() {
   if(test_uvw_to_xyz()) n++;
   if(test_xyz_to_uvw()) n++;
   if(test_julian_date()) n++;
+  if(test_jd_from_date()) n++;
+  if(test_jd_to_date()) n++;
   if(test_parse_date()) n++;
   if(test_parse_iso_date()) n++;
   if(test_date()) n++;
@@ -2247,6 +2489,15 @@ int main() {
   if(test_print_hms()) n++;
   if(test_print_dms()) n++;
 
+  if(test_time_lst()) n++;
+
+  if(test_novas_make_planet_orbit()) n++;
+  if(test_novas_make_moon_orbit()) n++;
+  if(test_approx_heliocentric()) n++;
+  if(test_approx_sky_pos()) n++;
+  if(test_moon_phase()) n++;
+  if(test_next_moon_phase()) n++;
+
   if(n) fprintf(stderr, " -- FAILED %d tests\n", n);
   else fprintf(stderr, " -- OK\n");
 
